import { to } from 'await-to-js';
import { getToken, removeToken, setToken } from '@/utils/auth';
import { login as loginApi, logout as logoutApi, getInfo as getUserInfo } from '@/api/login';
import defAva from '@/assets/images/profile.jpg';
import store from '@/store';

export const useUserStore = defineStore('user', () => {
  const token = ref(getToken());
  const name = ref('');
  const nickname = ref('');
  const userId = ref('');
  const tenantId = ref('');
  const avatar = ref('');
  const roles = ref([]); // 用户角色编码集合 → 判断路由权限
  const permissions = ref([]); // 用户权限编码集合 → 判断按钮权限
  // 空闲超时时间
  const idleTimeout = ref(60 * 60 * 1000);
  // 最后操作时间
  const lastRequestTime = ref();
  /**
   * 登录
   * @param userInfo
   * @returns
   */
  const login = async (userInfo) => {
    const [err, res] = await to(loginApi(userInfo));
    if (res) {
      const data = res.data;
      setToken(data.access_token);
      token.value = data.access_token;
      return Promise.resolve();
    }
    return Promise.reject(err);
  };

  // 获取用户信息
  const getInfo = async () => {
    const [err, res] = await to(getUserInfo());
    if (res) {
      const data = res.data;
      const user = data.user;
      const profile = user.avatar == '' || user.avatar == null ? defAva : user.avatar;

      if (data.roles && data.roles.length > 0) {
        // 验证返回的roles是否是一个非空数组
        roles.value = data.roles;
        permissions.value = data.permissions;
      } else {
        roles.value = ['ROLE_DEFAULT'];
      }
      name.value = user.userName;
      nickname.value = user.nickName;
      avatar.value = profile;
      userId.value = user.userId;
      tenantId.value = user.tenantId;
      return Promise.resolve();
    }
    return Promise.reject(err);
  };

  // 注销
  const logout = async () => {
    await logoutApi();
    token.value = '';
    roles.value = [];
    permissions.value = [];
    removeToken();
  };

  const setAvatar = (value) => {
    avatar.value = value;
  };

  let interval;
  const setTimeoutInterval = () => {
    if (!!interval) {
      toClearInterval();
    }
    lastRequestTime.value = new Date().getTime();
    interval = setInterval(() => {
      const currentTime = new Date().getTime();
      if (currentTime - lastRequestTime.value > idleTimeout.value) {
        toClearInterval();
        alert('登录超时,请重新登录');
        logout().then(() => {
          location.href = import.meta.env.VITE_APP_CONTEXT_PATH + '#' + 'index';
        });
      }
    }, 5000);
  };
  const updateLastRequestTime = () => {
    lastRequestTime.value = new Date().getTime();
  };
  const toClearInterval = () => {
    if (interval) {
      clearInterval(interval);
      interval = null;
    }
  };
  return {
    userId,
    tenantId,
    token,
    nickname,
    avatar,
    roles,
    permissions,
    login,
    getInfo,
    logout,
    setAvatar,
    setTimeoutInterval,
    updateLastRequestTime
  };
});

export default useUserStore;
// 非setup
export function useUserStoreHook() {
  return useUserStore(store);
}
